"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AccessoryInfoServiceInitializer = exports.SmartSpeakerServiceInitializer = exports.BaseServiceInitializer = void 0;
class BaseServiceInitializer {
    constructor(logger, hap, alexaRemote) {
        this.logger = logger;
        this.hap = hap;
        this.alexaRemote = alexaRemote;
        this.accessories = new Map();
    }
    initialize(accessory, device, media) {
        this.init(this.getService(accessory), device, media);
        this.accessories.set(device.serialNumber, accessory);
        this.alexaRemote.on('command', command => this.onCommand(command));
    }
    getService(accessory) {
        const serviceType = this.getServiceType();
        let service = accessory.getService(serviceType);
        if (!service) {
            service = accessory.addService(serviceType);
        }
        return service;
    }
    onSet(service, device, characteristic, getCommand, getCommandValue) {
        service
            .getCharacteristic(characteristic)
            .on("set" /* SET */, (value, callback) => {
            try {
                const command = 'function' === typeof getCommand ? getCommand(value) : getCommand;
                const commandValue = 'function' === typeof getCommandValue ? getCommandValue(value) : getCommandValue;
                this.alexaRemote.sendCommand(device.serialNumber, command, commandValue, e => {
                    if (e) {
                        const message = `Failed to send ${command} to set ${characteristic.name} to ${JSON.stringify(commandValue)}.`;
                        this.logger.error(message, e);
                        return callback(new Error(message));
                    }
                    return callback(undefined);
                });
            }
            catch (e) {
                this.logger.error(`Uncaught error on setting ${characteristic.name}: ${JSON.stringify(e)}`);
                callback(e);
            }
        });
    }
    onCommand(command) {
        if (this.isDopplerCommand(command)) {
            const accessory = this.accessories.get(command.payload.dopplerId.deviceSerialNumber);
            if (accessory) {
                this.onServiceCommand(this.getService(accessory), command);
            }
        }
    }
    isDopplerCommand(command) {
        return 'dopplerId' in command.payload;
    }
}
exports.BaseServiceInitializer = BaseServiceInitializer;
class SmartSpeakerServiceInitializer extends BaseServiceInitializer {
    getServiceType() {
        return this.hap.Service.SmartSpeaker;
    }
    init(service, device, media) {
        this.setCurrentMediaState(service, media.currentState);
        this.setTargetMediaState(service, media.currentState);
        this.setVolume(service, media.volume);
        this.setMute(service, media.muted);
        const HapCharacteristic = this.hap.Characteristic;
        const TargetMediaState = HapCharacteristic.TargetMediaState;
        const Volume = HapCharacteristic.Volume;
        const Mute = HapCharacteristic.Mute;
        this.onSet(service, device, TargetMediaState, value => {
            switch (value) {
                case TargetMediaState.PAUSE:
                case TargetMediaState.STOP:
                    return 'pause';
                case TargetMediaState.PLAY:
                    return 'play';
            }
            throw new Error(`Unexpected value: ${JSON.stringify(value)}`);
        });
        this.onSet(service, device, Volume, 'volume', value => value);
        this.onSet(service, device, Mute, 'mute', value => {
            return { mute: value };
        });
    }
    onServiceCommand(service, command) {
        if (this.isVolumeChange(command)) {
            if (null !== command.payload.volumeSetting) {
                this.setVolume(service, command.payload.volumeSetting);
            }
            if (null !== command.payload.isMuted) {
                this.setMute(service, command.payload.isMuted);
            }
        }
        else if (this.isAudioPlayerState(command)) {
            this.setCurrentMediaState(service, command.payload.audioPlayerState);
            this.setTargetMediaState(service, command.payload.audioPlayerState);
        }
    }
    setVolume(service, volume) {
        service.getCharacteristic(this.hap.Characteristic.Volume).setValue(volume);
    }
    setMute(service, isMuted) {
        service.getCharacteristic(this.hap.Characteristic.Mute).setValue(isMuted);
    }
    setCurrentMediaState(service, state) {
        const CurrentMediaState = this.hap.Characteristic.CurrentMediaState;
        let currentMediaState;
        switch (state) {
            case 'PLAYING':
                currentMediaState = CurrentMediaState.PLAY;
                break;
            case 'INTERRUPTED':
                currentMediaState = CurrentMediaState.PAUSE;
                break;
            case 'FINISHED':
            default:
                currentMediaState = CurrentMediaState.STOP;
                break;
        }
        service.getCharacteristic(CurrentMediaState).setValue(currentMediaState);
    }
    setTargetMediaState(service, state) {
        const TargetMediaState = this.hap.Characteristic.TargetMediaState;
        let targetMediaState;
        switch (state) {
            case 'PLAYING':
                targetMediaState = TargetMediaState.PLAY;
                break;
            case 'INTERRUPTED':
                targetMediaState = TargetMediaState.PAUSE;
                break;
            case 'FINISHED':
            default:
                targetMediaState = TargetMediaState.STOP;
                break;
        }
        service.getCharacteristic(TargetMediaState).setValue(targetMediaState);
    }
    isVolumeChange(command) {
        return command.command === "PUSH_VOLUME_COMMAND" /* PUSH_VOLUME_COMMAND */;
    }
    isAudioPlayerState(command) {
        return command.command === "PUSH_AUDIO_PLAYER_STATE" /* PUSH_AUDIO_PLAYER_STATE */;
    }
}
exports.SmartSpeakerServiceInitializer = SmartSpeakerServiceInitializer;
class AccessoryInfoServiceInitializer extends BaseServiceInitializer {
    getServiceType() {
        return this.hap.Service.AccessoryInformation;
    }
    init(service, device, media) {
        // TODO: const identify = service.getCharacteristic(this.Characteristic.Identify);
        const manufacturer = service.getCharacteristic(this.hap.Characteristic.Manufacturer);
        const model = service.getCharacteristic(this.hap.Characteristic.Model);
        const name = service.getCharacteristic(this.hap.Characteristic.Name);
        const serialNumber = service.getCharacteristic(this.hap.Characteristic.SerialNumber);
        const firmwareRevision = service.getCharacteristic(this.hap.Characteristic.FirmwareRevision);
        manufacturer.setValue('Amazon.com');
        model.setValue(device.deviceType);
        firmwareRevision.setValue(device.softwareVersion);
        name.setValue(device.accountName);
        serialNumber.setValue(device.serialNumber);
    }
    onServiceCommand(service, command) {
        // TODO: Handle device name changes and software updates.
    }
}
exports.AccessoryInfoServiceInitializer = AccessoryInfoServiceInitializer;
